import EventEmitter from "events";

import { ListenerRss, ListenerRSSInfos } from "listener-rss";
import { SqliteTools } from "./sqlite-tools";

/**
 * Permit to manage a ListenerRSS array, data storage and event aggregation
 */
export class ListenerRssAggregator extends EventEmitter {
  private listenerArray: Map<string, ListenerRss> = new Map(); // renomer listenerMap
  private looprunning: boolean = false;

  private sqliteDb: SqliteTools;

  constructor(path: string) {
    super();
    this.sqliteDb = new SqliteTools(path);
    this.load();
  }

  async getAllConfigs(): Promise<ListenerRSSInfos[]> {
    return await this.sqliteDb.fetchAll();
  }

  private async load() {
    const configs = await this.getAllConfigs();

    configs.forEach((config) => {
      this.addNewListener(config);
    });
  }

  private addNewListener(info: ListenerRSSInfos): ListenerRss {
    const newListener = new ListenerRss(info);
    this.listenerArray.set(newListener.address, newListener);

    newListener.on("update", (obj) => this.emit("update", obj));
    newListener.on("newEntries", (obj) => this.emit("newEntries", obj));
    newListener.on("error", (err) => this.emit("error", err));

    return newListener;
  }

  private removeOldListener(adr: string): void {
    const oldListener = this.listenerArray.get(adr);

    if (!oldListener) return;

    oldListener.stop();
    oldListener
      .removeAllListeners("update")
      .removeAllListeners("newEntries")
      .removeAllListeners("error");
    this.listenerArray.delete(adr);
  }

  async registerListener(info: ListenerRSSInfos) {
    if (this.listenerArray.has(info.address)) return;

    const listener = this.addNewListener(info);
    await this.sqliteDb.insertListener(listener);
  }

  async unregisterListener(adr: string): Promise<void> {
    if (!this.listenerArray.has(adr)) return;

    this.removeOldListener(adr);
    await this.sqliteDb.deleteListener(adr);
  }

  async saveOverride(expectedConfig: ListenerRSSInfos[]) {
    const actualConfig = await this.getAllConfigs();

    for (const newItem of expectedConfig.filter(
      (item) => !actualConfig.includes(item)
    ))
      this.registerListener(newItem);

    for (const oldItem of actualConfig.filter(
      (item) => !expectedConfig.includes(item)
    ))
      this.unregisterListener(oldItem.address);
  }

  startAll(): void {
    if (this.looprunning) return;

    this.looprunning = true;
    for (const item of this.listenerArray.values()) item.start();
  }

  stopAll(): void {
    if (!this.looprunning) return;

    this.looprunning = false;
    for (const item of this.listenerArray.values()) item.stop();
  }
}
